#!/usr/bin/env python
# coding: utf-8
from functools import partial
import logging

from babel import Locale, UnknownLocaleError, default_locale
import click
import tornado.web
import tornado.httpserver
import tornado.ioloop

from .core import CriticApp
import critics


logger = logging.getLogger('critics')


@click.command()
@click.option('--ios', multiple=True, help='ios app id, e.g. 122434343')
@click.option('--ios-channel', help='Slack channel for ios notifications, optional')
@click.option('--android', multiple=True, help='Android app name, e.g. "com.rovio.angrybirds"')
@click.option('--android-channel', help='Slack channel for android notifications, optional')
@click.option('--language', multiple=True, help='ISO 639-1 languages of review [default: system locale]')
@click.option('--slack-webhook', help='Slack webhook absolute URL, required')
@click.option('--parse-max-entries', default=10, help='Number of feed entries to look into [default: 10]')
@click.option('--beat', default=300, help='Number of seconds between polling feed [default: 300]')
@click.option('--verbose/--short', default=False)
@click.option('--notify/--no-notify', default=True)
@click.option('--persist/--no-persist', default=True)
@click.option('--model', type=click.Path(), default='reviews.json')
@click.option('--daemonize/--run-once', default=True)
@click.option('--version', is_flag=True)
def cli(**settings):
    """Notify about new reviews in AppStore and Google Play in slack.

       Launch command using supervisor or using screen/tmux/etc.
       Reviews are fetched for multiple apps and languages in --beat=300 interval.
    """

    setup_logging(settings)
    settings = setup_languages(settings)
    app = CriticApp(**settings)

    if settings['version']:
        click.echo('Version %s' % critics.__version__)
        return
    if not (settings['ios'] or settings['android']):
        click.echo('Please choose either --ios or --android')
        return

    loop = tornado.ioloop.IOLoop.instance()

    if app.load_model():
        logger.debug('Model loaded OK, not skipping notify on first run')
        notify = True
    else:
        notify = False

    if settings['ios']:
        logger.info('Tracking IOS apps: %s', ', '.join(settings['ios']))
        itunes = tornado.ioloop.PeriodicCallback(partial(app.poll_store, 'ios'),
                                                 1000 * settings['beat'], loop)
        itunes.start()
    if settings['android']:
        logger.info('Tracking Android apps: %s', ', '.join(settings['android']))
        google_play = tornado.ioloop.PeriodicCallback(partial(app.poll_store, 'android'),
                                                      1000 * settings['beat'], loop)
        google_play.start()

    echo_channel_map(settings)

    if settings['ios']:
        app.poll_store('ios', notify=notify)
    if settings['android']:
        app.poll_store('android', notify=notify)

    if settings['daemonize']:
        loop.start()


def setup_logging(settings):
    handler = logging.StreamHandler()
    logger.addHandler(handler)
    if settings['verbose']:
        logger.setLevel(logging.DEBUG)
        handler.setFormatter(logging.Formatter('[%(asctime)s] %(message)s', "%Y-%m-%d %H:%M:%S"))
    else:
        logger.setLevel(logging.INFO)
    logger.propagate = False


def setup_languages(settings):
    if not settings['language']:
        try:
            settings['language'] = [default_locale()[:2]]
        except ValueError:
            settings['language'] = ['en']

    languages = []
    language_names = []
    for lang_code in settings['language']:
        try:
            language_names.append(Locale(lang_code).english_name)
            languages.append(lang_code)
        except UnknownLocaleError:
            raise click.ClickException('Unknown language code: %s' % lang_code)

    logger.info('Languages: %s', ', '.join(language_names))
    return settings


def echo_channel_map(settings):
    if not settings['slack_webhook']:
        return
    channel_map = ''
    if settings['ios'] and settings['ios_channel']:
        channel_map += ' IOS -> %s  ' % settings['ios_channel']
    if settings['android'] and settings['android_channel']:
        channel_map += ' Android -> %s' % settings['android_channel']
    if channel_map:
        logger.info('Transport: slack channels:%s' % channel_map)



# coding: utf-8
from collections import namedtuple
import json
import datetime
import logging
import os
import re
from time import mktime

import feedparser
import requests
from lxml import html

from .compat import python_2_unicode_compatible


logger = logging.getLogger('critics')
timeout = os.environ.get('CRITICS_TIMEOUT', 5)


@python_2_unicode_compatible
class Review(namedtuple('Review',
                        ['id', 'platform', 'title', 'rating', 'summary', 'url',
                         'author', 'date', 'language', 'version'])):
    __slots__ = ()

    def __str__(self):
        return (u'Review (%s):\ntitle=%s\nrating=%s\nsummary=%s\nurl=%s\n'
                u'author=%s\ndate=%s\nlanguage=%s\nversion=%s' % (
                    self.id,
                    self.title,
                    self.rating,
                    self.summary,
                    self.url,
                    self.author,
                    self.date,
                    self.language,
                    self.version
                ))


def get_ios_reviews(app_id, language, limit=100):
    url = 'https://itunes.apple.com/%(language)srss/customerreviews/id=%(app_id)s/sortBy=mostRecent/xml' % {
        'language': '%s/' % language if language else '', 'app_id': app_id}
    response = requests.get(url, headers={'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_1)'},
                            timeout=timeout)
    response.encoding = 'utf-8'  # avoid chardet not guessing correctly
    feed = feedparser.parse(response.text)
    reviews = [Review(
        id=entry.id,
        platform='ios',
        title=entry.title,
        rating=int(entry.im_rating),
        summary=entry.summary,
        url=entry.href,
        author=entry.author,  # author url: entry.href
        date=datetime.datetime.fromtimestamp(mktime(entry.updated_parsed)),
        language=language,
        version=entry.im_version
    ) for entry in feed['entries'][1:1 + limit]]
    return reviews


def get_android_reviews(app_id, language, limit=100):
    url = 'https://play.google.com/store/getreviews'
    payload = {'xhr': 1, 'id': app_id, 'reviewSortOrder': 0, 'pageNum': 0, 'reviewType': 0}
    if language:
        payload['hl'] = language
    response = requests.post(url, data=payload, timeout=timeout)
    json_source = response.text[response.text.find('['):]
    response_as_json = json.loads(json_source)
    try:
        response_as_html = response_as_json[0][2]
    except IndexError:
        logger.error('Unexpected json for app_id=%s', app_id)
        return []

    utf8_parser = html.HTMLParser(encoding='utf-8')
    doc = html.fromstring(response_as_html.encode('utf-8'), parser=utf8_parser)
    reviews_html = doc.cssselect('.single-review')

    def get_rating_from_html(review_html):
        star_style = review_html.cssselect('.current-rating')[0].get('style')  # e.g. 'width: 20%'
        return int(re.search('(\d+)%', star_style).group(1)) / 20

    reviews = [Review(
        id=review_html.cssselect('.review-header')[0].get('data-reviewid'),
        platform='android',
        title=review_html.cssselect('.review-body .review-title')[0].text_content().strip(),
        rating=get_rating_from_html(review_html),
        summary=review_html.cssselect('.review-body .review-title')[0].tail.strip(),
        url='https://play.google.com' + review_html.cssselect('.reviews-permalink')[0].get('href'),
        author=review_html.cssselect('.review-header .author-name')[0].text_content().strip(),
        date=review_html.cssselect('.review-header .review-date')[0].text_content().strip(),
        language=language,
        version=None
    ) for review_html in reviews_html[:limit]]
    return reviews



# coding: utf-8
from os.path import dirname, join, abspath

from babel import default_locale
from babel.support import Translations


translations_dir = abspath(join(dirname(__file__), 'locale'))


def get_locale():
    return default_locale()


def get_language():
    language = Translations.load(translations_dir)
    return language



# coding: utf-8
import json
import datetime

from critics.i18n import get_language, get_locale
from critics.parsers import Review
from critics.transport import post2slack


try:
    from urllib.parse import parse_qs
except ImportError:
    from urlparse import parse_qs

import responses


def patch_locale(monkeypatch, lang):
    monkeypatch.setenv('LANGUAGE', lang)
    monkeypatch.setattr('critics.transport.language', get_language())
    monkeypatch.setattr('critics.transport._', get_language().ugettext)
    monkeypatch.setattr('critics.transport.locale', get_locale())


@responses.activate
def test_slack(monkeypatch):
    patch_locale(monkeypatch, 'en')
    responses.add(responses.POST, 'http://slack.hook/')

    post2slack([Review(
        id=u'gp:AOqpTOGVTpUWQyZJhCbF6dpGfgWnmaj992am2upoCj8Iur1YoNKubAlN_W_BzRYHGh7pdxAe-HDIdOUEvaRfpw',
        platform='android',
        title=u'',
        rating=1,
        summary=u'dffdf\nfgf4tt',
        url=u'http://www',
        author=u'Someauthor',
        date=datetime.datetime(2015, 1, 1, 6),
        language='ru',
        version=None
    )], 'http://slack.hook/', channel=None)

    assert json.loads(parse_qs(responses.calls[0].request.body)['payload'][0]) == {
        u'username': u'Critic',
        u'text': u'There is 1 new review in Google Play',
        u'attachments': [{
            u'color': u'#CC2525',
            u'text': u'★☆☆☆☆\ndffdf\nfgf4tt\n\n_Someauthor_, 1 January 2015 06:00 ',
            u'title_link': u'http://www',
            u'mrkdwn_in': [u'text'],
            u'title': u'-'}
        ],
        u'icon_emoji': u':feelsgood:'}


@responses.activate
def test_slack_locale(monkeypatch):
    patch_locale(monkeypatch, 'ru')
    responses.add(responses.POST, 'http://slack.hook/')

    post2slack([Review(
        id=u'34e56344☃',
        platform='ios',
        title=u'Great app! ♡',
        rating=5,
        summary=u'NOO̼O​O NΘ stop the an​*̶͑̾̾​̅ͫ͏̙̤g͇̫͛͆̾ͫ̑͆l͖͉̗̩̳̟̍ͫͥͨ',
        url=u'http://www',
        author=u'Here comes more BS',
        date=datetime.datetime(2015, 1, 1, 6),
        language=None,
        version='2.1.3'
    )], 'http://slack.hook/', channel=None)

    assert len(responses.calls) == 1
    request = responses.calls[0].request
    assert request.url == 'http://slack.hook/'
    assert json.loads(parse_qs(request.body)['payload'][0]) == {
        u'username': u'Критик',
        u'text': u'В AppStore 1 новый отзыв',
        u'attachments': [{
            u'color': u'#30E80C',
            u'text': u'★★★★★\nNOO̼O​O NΘ stop the an​*̶͑̾̾​̅ͫ͏̙̤g͇̫͛͆̾ͫ̑͆l͖͉̗̩̳̟̍ͫͥͨ\n\n_Here comes more BS_, 1 января 2015 06:00 [2.1.3]',  # noqa
            u'title_link': u'http://www',
            u'mrkdwn_in': [u'text'],
            u'title': u'Great app! \u2661'}
        ],
        u'icon_emoji': u':godmode:'}



# coding: utf-8
import codecs
import datetime
import responses
from critics.parsers import get_ios_reviews, get_android_reviews


@responses.activate
def test_ios():
    responses.add(responses.GET, 'https://itunes.apple.com/fr/rss/customerreviews/id=123/sortBy=mostRecent/xml',
                  body=codecs.open('tests/fixtures/itunes_fr.example', encoding='utf-8').read(),
                  content_type='application/xml; charset=UTF-8')
    reviews = get_ios_reviews(123, language='fr')

    assert len(reviews) == 50

    review = reviews[0]
    assert review.id == u'1163882289'
    assert review.title == u'Leon, super jeu'
    assert review.rating == 5
    assert review.summary == u'Moi qui ne suis pas n grand amateur, j\'ai adoré ce jeu malin et intuitif.'
    assert review.url == u'https://itunes.apple.com/fr/reviews/id270119737'
    assert review.author == u'Justine M18'
    assert review.date == datetime.datetime(2015, 3, 5, 20, 5)
    assert review.version == '1.7'

    assert str(review) == """Review (1163882289):
title=Leon, super jeu
rating=5
summary=Moi qui ne suis pas n grand amateur, j'ai adoré ce jeu malin et intuitif.
url=https://itunes.apple.com/fr/reviews/id270119737
author=Justine M18
date=2015-03-05 20:05:00
language=fr
version=1.7"""


@responses.activate
def test_android_ru():
    responses.add(responses.POST, 'https://play.google.com/store/getreviews',
                  body=codecs.open('tests/fixtures/gp_ru.example', encoding='utf-8').read(),
                  content_type='application/json; charset=UTF-8')
    reviews = get_android_reviews('com.skype.raider', language=None, limit=10)

    assert 'hl=en' not in responses.calls[0].request.body
    assert len(reviews) == 10

    review = reviews[0]
    assert review.id == u'gp:AOqpTOGVTpUWQyZJhCbF6dpGfgWnmaj992am2upoCj8Iur1YoNKubAlN_W_BzRYHGh7pdxAe-HDIdOUEvaRfpw'
    assert review.title == u'Не работает'
    assert review.rating == 1
    assert review.summary == u'Не работает видео связь на nvidia shield'
    assert review.url == u'https://play.google.com/store/apps/details?id=com.skype.raider&reviewId=Z3A6QU9xcFRPR1ZUcFVXUXlaSmhDYkY2ZHBHZmdXbm1hajk5MmFtMnVwb0NqOEl1cjFZb05LdWJBbE5fV19CelJZSEdoN3BkeEFlLUhESWRPVUV2YVJmcHc'  # noqa
    assert review.author == u'Polar Alexander'
    assert review.date == u'7 марта 2015 г.'
    assert review.version is None

    assert reviews[1].rating == 2


@responses.activate
def test_android_en():
    responses.add(responses.POST, 'https://play.google.com/store/getreviews',
                  body=codecs.open('tests/fixtures/gp_en.example', encoding='utf-8').read(),
                  content_type='application/json; charset=UTF-8')
    reviews = get_android_reviews('com.skype.raider', language='en', limit=10)

    assert 'hl=en' in responses.calls[0].request.body
    assert len(reviews) == 10

    review = reviews[0]
    assert review.id == u'gp:AOqpTOFuJk3lGt-xaYQLrp03KTe8eg7qs7Bm9wkUZQXQ1b2knLXIdpq1e9ZPM_AwCEa3szAZH9DDqWPbVE5qBQ'
    assert review.title == u''
    assert review.summary == u'I like skype'
    assert review.date == u'March 14, 2015'
    assert review.version is None



import os
import json
import logging
import tornado.web
from .parsers import get_android_reviews, get_ios_reviews
from .transport import post2slack


logger = logging.getLogger('critics')


class CriticApp(tornado.web.Application):

    def __init__(self, *args, **kwargs):
        super(CriticApp, self).__init__(*args, **kwargs)
        self.reviews = {
            'android': set(),
            'ios': set(),
        }
        self.fetchers = {
            'android': get_android_reviews,
            'ios': get_ios_reviews
        }
        self.notifiers = {
            'slack': post2slack
        }

    def poll_store(self, platform, notify=True):
        for app_id in self.settings.get(platform):
            for language in self.settings['language']:
                self.poll_store_single_app(platform, app_id, language, notify)

    def poll_store_single_app(self, platform, app_id, language, notify):
        if platform not in self.fetchers.keys():
            raise Exception('Unknown platform %s', platform)
        fetcher = self.fetchers[platform]

        new_reviews = []
        parsed_review_ids = self.reviews[platform]
        logging.debug('parsed_review_ids = %s', parsed_review_ids)
        reviews = fetcher(app_id=app_id,
                          language=language,
                          limit=self.settings.get('parse_max_entries', None))
        for review in reviews:
            if review.id in parsed_review_ids:
                continue
            logger.debug(review)
            parsed_review_ids.add(review.id)
            new_reviews.append(review)

        logger.info('%(platform)s: %(app_id)s: %(language)s: Fetched %(num_reviews)s reviews, %(new_reviews)s new',
                    {'platform': platform, 'app_id': app_id, 'language': language,
                     'num_reviews': len(reviews), 'new_reviews': len(new_reviews)})

        if new_reviews:
            self.send_messages(new_reviews, platform, notify)

    def send_messages(self, new_reviews, platform, notify):
        self.save_model()
        channel = self.settings.get('%s_channel' % platform, None)
        if not notify:
            logger.debug('%s: Skip notification on first run', platform)
        elif not self.settings.get('notify', True):
            logger.debug('%(platform)s: Faking %(num)s notifications to %(channel)s', {
                'platform': platform, 'num': len(new_reviews), 'channel': channel
            })
        else:
            self.notifiers['slack'](new_reviews, self.settings['slack_webhook'], channel)
            logging.debug(new_reviews)

    def load_model(self):
        if not self.settings['persist']:
            return False
        if os.path.exists(self.settings['model']):
            logger.debug('Loading state')
            try:
                with open(self.settings['model']) as fp:
                    self.reviews = json.load(fp, object_hook=as_set)
            except Exception as e:
                logger.error('Problem loading prev. results from file: %s', e)
            else:
                return True
        return False

    def save_model(self):
        if not self.settings['persist']:
            return False
        logger.debug('Saving state')
        try:
            json.dump(self.reviews, open(self.settings['model'], 'w'), cls=SetEncoder, indent=4)
        except Exception as e:
            logger.error('Problem saving results to file: %s', e)
            return False
        else:
            return True


class SetEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, set):
            return {'python_object': 'set', 'value': list(obj)}
        return json.JSONEncoder.default(self, obj)


def as_set(dct):
    if 'python_object' in dct:
        return set(dct['value'])
    return dct



# coding: utf-8
import json
from mock import Mock
from critics.core import CriticApp
from critics.parsers import Review


def test_poll_store(tmpdir):
    fakemodel = tmpdir.join("fakemodel.json")
    app = CriticApp(ios=['app1', 'app2'], language=['ru'], persist=True, slack_webhook='http://www')
    app.model_file = fakemodel.strpath
    fake_fetcher = Mock(return_value=[Review(
        id=u'xfkew4ytwqqddid:2e22wdad',
        platform='ios',
        title=u'Great app! ♡',
        rating=2,
        summary=u'Here be\nDragons!',
        url=u'http://www',
        author=u'Here comes more BS',
        date='bull',
        language='en',
        version=None
    )])
    app.fetchers['ios'] = fake_fetcher
    fake_notifier = Mock()
    app.notifiers['slack'] = fake_notifier
    app.poll_store(platform='ios')

    assert fake_fetcher.call_count == 2
    assert fake_notifier.call_count == 1
    assert len(app.reviews['ios']) == 1

    fake_fetcher.reset_mock()
    fake_notifier.reset_mock()
    app.poll_store(platform='ios')
    assert fake_fetcher.call_count == 2
    assert fake_notifier.call_count == 0
    assert len(app.reviews['ios']) == 1


def test_load_model():
    app = CriticApp(persist=False)
    assert not app.load_model()
    assert app.reviews['ios'] == set()
    assert app.reviews['android'] == set()

    app = CriticApp(persist=True, model='tests/fixtures/model.json')
    assert app.load_model()
    assert app.reviews['ios'] == {'123'}
    assert app.reviews['android'] == {'xxx', 'yyyy'}


def test_save_model(tmpdir):
    fakemodel = tmpdir.join("fakemodel.json")
    app = CriticApp(persist=False)

    assert not app.save_model()
    assert not fakemodel.check()

    app = CriticApp(persist=True, model=fakemodel.strpath)

    assert app.save_model()
    assert fakemodel.check()
    assert json.load(fakemodel) == {'android': {'python_object': 'set', 'value': []},
                                    'ios': {'python_object': 'set', 'value': []}}

    app.reviews['ios'].add('678')
    app.reviews['android'].add('qqq')
    assert app.save_model()
    assert fakemodel.check()
    assert json.load(fakemodel) == {'android': {'python_object': 'set', 'value': ['qqq']},
                                    'ios': {'python_object': 'set', 'value': ['678']}}



# coding: utf-8
import json

from click.testing import CliRunner
import pytest

from critics import envvar_prefix
from critics.commands import cli


@pytest.mark.functional
def test_args(tmpdir):
    model = tmpdir.join("reviews.json")
    runner = CliRunner()
    result = runner.invoke(cli, ['--ios', '343200656',
                                 '--android', 'com.rovio.angrybirds',
                                 '--language', 'ru',
                                 '--model', model.strpath,
                                 '--run-once'],
                           env={'CRITICS_SLACK_WEBHOOK': 'http://httpbin.org/post',
                                'CRITICS_IOS_CHANNEL': 'ios',
                                'CRITICS_ANDROID_CHANNEL': 'android'},
                           auto_envvar_prefix=envvar_prefix)
    assert result.exit_code == 0

    # iOS rss feed returns either 10 or 0 reviews - probably a bug on Apple side,
    # sometimes the feed is just empty.
    num_reviews_ios = len(json.loads(model.read())['ios']['value'])
    num_reviews_android = len(json.loads(model.read())['android']['value'])

    assert num_reviews_ios in [0, 10]
    assert num_reviews_android == 10
    assert result.output == ('Languages: Russian\n'
                             'Tracking IOS apps: 343200656\n'
                             'Tracking Android apps: com.rovio.angrybirds\n'
                             'Transport: slack channels: IOS -> ios   Android -> android\n'
                             'ios: 343200656: ru: Fetched %s reviews, %s new\n' % (num_reviews_ios, num_reviews_ios) +
                             'android: com.rovio.angrybirds: ru: Fetched 10 reviews, 10 new\n')


@pytest.mark.functional
def test_mutiple_languages(tmpdir):
    model = tmpdir.join("reviews.json")
    runner = CliRunner()
    result = runner.invoke(cli, ['--android', 'com.rovio.angrybirds',
                                 '--android', 'com.rovio.angrybirdsstarwarsii.ads',
                                 '--model', model.strpath,
                                 '--run-once', '--no-notify'],
                           env={'CRITICS_LANGUAGE': 'ru en'},
                           auto_envvar_prefix=envvar_prefix)
    assert result.exit_code == 0
    assert result.output == ('Languages: Russian, English\n'
                             'Tracking Android apps: com.rovio.angrybirds, com.rovio.angrybirdsstarwarsii.ads\n'
                             'android: com.rovio.angrybirds: ru: Fetched 10 reviews, 10 new\n'
                             'android: com.rovio.angrybirds: en: Fetched 10 reviews, 10 new\n'
                             'android: com.rovio.angrybirdsstarwarsii.ads: ru: Fetched 10 reviews, 10 new\n'
                             'android: com.rovio.angrybirdsstarwarsii.ads: en: Fetched 10 reviews, 10 new\n')



#!/usr/bin/env python
# coding: utf-8
import os
import re
import sys


try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


from setuptools.command.test import test as TestCommand


class PyTest(TestCommand):
    user_options = [('pytest-args=', 'a', "Arguments to pass to py.test")]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.pytest_args = []

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import pytest
        errno = pytest.main(self.pytest_args)
        sys.exit(errno)


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.search("__version__ = ['\"]([^'\"]+)['\"]", init_py).group(1)


with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('CHANGELOG.rst') as history_file:
    history = history_file.read()

requirements = [
    'requests>=2.6',
    'tornado>=4.1',
    'lxml>=3.4.2',
    'cssselect>=0.9.1',
    'feedparser>=5.1.3',
    'click>=3.3',
    'Babel>=1.3',
]

test_requirements = [
    'pytest>=2.6.4',
    'pytest-cov>=1.8.1',
    'responses>=0.3.0'
]

setup(
    name='critics',
    version=get_version('critics'),
    description="Mobile reviews aggregator",
    long_description=readme + '\n\n' + history,
    author="Ilya Baryshev",
    author_email='baryshev@gmail.com',
    url='https://github.com/coagulant/critics',
    packages=['critics'],
    package_dir={'critics': 'critics'},
    include_package_data=True,
    install_requires=requirements,
    entry_points="""
        [console_scripts]
        critics=critics:main
    """,
    license="BSD",
    zip_safe=False,
    keywords='critics',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
    test_suite='tests',
    tests_require=test_requirements,
    cmdclass={'test': PyTest},
)



# coding: utf-8
import json
import datetime
import math

from babel.dates import format_datetime
import requests

from .i18n import get_language, get_locale


locale = get_locale()
language = get_language()
_ = language.ugettext


def post2slack(reviews, slack_url, channel):
    emoji = {
        1: ':feelsgood:',
        2: ':finnadie:',
        3: ':hurtrealbad:',
        4: ':suspect:',
        5: ':godmode:',
    }
    colors = {
        1: '#CC2525',
        2: '#CC2525',
        3: '#E8AD0C',
        4: '#30E80C',
        5: '#30E80C',
    }
    stars = {
        1: u'★☆☆☆☆',
        2: u'★★☆☆☆',
        3: u'★★★☆☆',
        4: u'★★★★☆',
        5: u'★★★★★'
    }

    if not reviews:
        return

    def get_date_string(date):
        if isinstance(date, datetime.datetime):
            return format_datetime(date, 'd MMMM yyyy hh:mm', locale=get_locale())
        else:
            return date

    average_rating = int(math.floor(sum(review.rating for review in reviews) / float(len(reviews))))
    platform = reviews[0].platform

    store = _('AppStore') if platform == 'ios' else _('Google Play')
    template = language.ungettext('There is %(num)d new review in %(store)s',
                                  'There are %(num)d new reviews in %(store)s', len(reviews))
    text = template % {'num': len(reviews), 'store': store}

    payload = {
        'attachments': [{
            'color': colors[review.rating],
            'title': review.title or u'-',
            'title_link': review.url,
            'text': u'{stars}\n{summary}\n\n_{author}_, {date} {version}'.format(
                stars=stars[review.rating],
                summary=review.summary,
                author=review.author,
                date=get_date_string(review.date),
                version=u'[%s]' % review.version if review.version else ''
            ),
            "mrkdwn_in": ["text"],
        } for review in reviews],
        'text': text,
        'username': _('Critic'),
        "icon_emoji": emoji[average_rating],
    }
    if channel:
        payload['channel'] = channel

    requests.post(slack_url, data={'payload': json.dumps(payload)})
